//
// Mesh Class
// 
// Right now just loads simple meshes with only verts and triangles
//
#define _CRT_SECURE_NO_DEPRECATE
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <math.h>
#include <stdio.h>

#include "vec3.h"
#include "matrix.h"
#include "mesh.h"
#include "renderInterface.h"
#include "intersection.h"

CMesh::CMesh()
{
	m_nbVerts = 0;
	m_nbTris = 0;
	m_pVerts = NULL;
	m_pTris = NULL;
}

CMesh::~CMesh()
{
	delete[] m_pVerts;
	delete[] m_pTris;
}

void CMesh::Load( char *sFilename )
{
	FILE *pFile = fopen( sFilename, "rb" );
	if ( pFile ) 
		{
		fread( &m_nbVerts, 4, 1, pFile );
		fread( &m_nbTris , 4, 1, pFile );
		
		m_pVerts = new CVec3[ m_nbVerts + 1 ];
		m_pTris = new unsigned int[ m_nbTris*3 + 1 ];
		
		fread( m_pVerts, sizeof(CVec3), m_nbVerts, pFile );
		fread( m_pTris, 4 * 3, m_nbTris, pFile );
		
		fclose( pFile );
		}
}

void CMesh::Render( CRenderer *pRender )
{
	pRender->DrawMesh( m_pVerts, m_pTris, m_nbTris );
}


//
// Test a line against a mesh
// returns the number of intersections
//
int CMesh::CheckLine( const CVec3 &LP1, const CVec3 &LP2 )
{
	CVec3 HitP;
	int nbHits = 0;
	
	for (int nTri = 0; nTri < m_nbTris; nTri++ )
		{
		int nV = nTri*3;
		bool bHit = CheckLineTri( LP2, LP1, m_pVerts[ m_pTris[ nV ] ], m_pVerts[ m_pTris[ nV+1 ] ], m_pVerts[ m_pTris[ nV+2 ] ], HitP );
		if ( bHit ) {
			nbHits++;
			// Render the point of intersection so we can see that everything's working right visually
			CRenderer::HighlightVert( HitP );
			}
		}
		
	return nbHits;
}